import { SBaseEditScene, SBaseLineEdit, SBaseTextEdit, SBaseImageEdit, SBaseExpainEdit, SBaseRectEdit } from "./../big-edit";
import { SPersagyImageEdit } from "./"
// import { SGraphItem } from "@persagy-web/graph/";
import { SGraphEdit } from "./../edit"
import { SMouseEvent } from "@persagy-web/base/lib";
import { SGraphSelectContainer } from "@persagy-web/graph";
import { SItemStatus } from "@persagy-web/big/lib/enums/SItemStatus";
import { uuid, rgbaNum } from "./../big-edit/until";

// 引入命令
import { SGraphAddCommand } from "./../edit/commands/SGraphAddCommand"
import { SColor } from '@persagy-web/draw/lib';
export class PTopoScene extends SBaseEditScene {
    constructor() {
        super()
        // // 选择绑定选额item事件
        this.selectContainer.connect("listChange", this, this.listChange);
    }

    /**
     * 选中返回的选中item回调方法
     *
     * @param   event   鼠标事件参数
     */
    listChange(list: any): void {
        this.emitChoice(list.itemList);
    }

    /**
     * 选中返回的选中item回调方法(用于场景的外部调用)
     *
     * @param   list   选中的item数组
    */
    emitChoice(list: any) {
    }

    /**
     * 鼠标左键按下
     *
     * @param   event   鼠标事件参数
     */
    onMouseDown(event: SMouseEvent): any {
        this.vueOnMouseDown(event) //外部调用
        if (this.editCmd == "EditBaseLine") {
            this.addLine(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBasetext") {
            this.addTextItem(event);
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBaseImage") {
            this.addImageItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBasePostil") {
            this.addExplainItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBasePolygon") {
            console.log('编辑多边形')
        } else if (this.editCmd == "EditBaseRect") {
            this.addRectItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBaseTriangle") {
            this.addTriangleItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "EditBaseCircle") {
            console.log('编辑圆')
        } else if (this.editCmd == "EditBaseArrows") {
            this.addArrowsItem(event)
            this.clearCmdStatus();
        } else if (this.editCmd == "") {
            super.onMouseDown(event);
        }
    }
    /**
     * 鼠标右键事件
     *
     * @param   event   鼠标事件参数
     */
    onContextMenu(event: SMouseEvent): boolean {
        if (!super.onContextMenu(event)) {
            this.getItem(null, [event])
        }
        return true
    }
    /////////////////////////////////////////////////////////////////////////////////////////////////
    //新增 item 类方法；
    /**
     * 新增基础类直线
     *
     * @param   event   鼠标事件参数
     */
    addLine(event: SMouseEvent): void {
        const data = {
            /** ID */
            // ID: uuid(),
            /** 名称  */
            Name: '基础直线',
            /** 图标（Image），线类型(Line) */
            Type: "Line",
            /** 缩放比例（Image），线类型(Line) */
            Scale: { X: 1, Y: 1, Z: 1 },
            /** 缩放比例（Image），线类型(Line) */
            Rolate: { X: 0, Y: 0, Z: 0 },
            /** 位置  */
            Pos: { X: 0, Y: 0 },
            /** 由应用自己定义  */
            Properties: {
                Type: "BaseLine"           // 自定义类型用于区分mark与node
            },
            Style: {
                Line: [{ X: event.x, Y: event.y }],
            }
        }
        const lineItem = new SBaseLineEdit(null, data);
        lineItem.status = SItemStatus.Create;
        this.addItem(lineItem);
        this.undoStack.push(new SGraphAddCommand(this, lineItem));
        lineItem.selectable = true;
        this.grabItem = lineItem;
        lineItem.connect("finishCreated", this, this.finishCreated);
        lineItem.connect("onContextMenu", this, this.getItem);
        if (this.view) {
            this.view.update();
        }
    }

    /**
     * 新增基础类文本
     *
     * @param   event   鼠标事件参数
     */
    addTextItem(event: SMouseEvent): void {
        const data = {
            /** ID */
            ID: uuid(),
            /** 名称  */
            Name: '基础文本',
            /** 图标 */
            Type: "Text",
            /** 位置  */
            Pos: { X: event.x, Y: event.y },
            /** 由应用自己定义  */
            Properties: {
                // IconUrl: require(`../../assets/images/t-text-hover.png`),
                Text: '请在右侧属性栏输入文字！',
                Color: "#646c73",
                Font: 14,
                BackgroundColor: "#f7f9facc",
                Type: "BaseText"           // 自定义类型用于区分mark与node
            },
            Style: {}
        }
        const item = new SBaseExpainEdit(null, data);
        item.moveTo(event.x, event.y);
        item.selectable = true;
        item.moveable = true;
        this.addItem(item);
        this.undoStack.push(new SGraphAddCommand(this, item));
        // this.Markers.push(item);
        this.grabItem = null;
        // this.focusItem = item;
        item.connect("onContextMenu", this, this.getItem);
        this.finishCreated(item);
        // this.scenceUpdate(this);
    }

    /**
     * 新增基础类注释
     *
     * @param   event   鼠标事件参数
     */
    addExplainItem(event: SMouseEvent): void {
        const data = {
            /** ID */
            ID: uuid(),
            /** 名称  */
            Name: '基础注释文本',
            /** 图标 */
            Type: "Text",
            /** 位置  */
            Pos: { X: event.x, Y: event.y },
            /** 由应用自己定义  */
            Properties: {
                // IconUrl: require(`../../assets/images/t-text-hover.png`),
                Text: '请在右侧属性栏输入文字！',
                Color: "#646c73",
                Font: 14,
                BackgroundColor: "#f7f9facc",
                Type: "BaseExplain",
            },
            Style: {}
        }
        const item = new SBaseTextEdit(null, data);
        item.moveTo(event.x, event.y);
        item.selectable = true;
        item.moveable = true;
        this.addItem(item);
        this.undoStack.push(new SGraphAddCommand(this, item));
        // this.Markers.push(item);
        this.grabItem = null;
        // this.focusItem = item;
        this.finishCreated(item);
        // this.scenceUpdate(this);
    }

    /**
     * 新增基础类图片
     *
     * @param   event   鼠标事件参数
     */
    addImageItem(event: SMouseEvent): void {
        const data = {
            /** ID */
            ID: uuid(),
            /** 名称  */
            Name: '基础图片',
            Num: 1,
            /** 图标（Image），线类型(Line) */
            Type: "Image",
            /** 位置  */
            Pos: { X: event.x, Y: event.y },
            /** 由应用自己定义  */
            Properties: {
                // IconUrl: require(`../../assets/images/t-img-hover.png`),
                StrokeColor: "#c0ccda",
                Url: '',
                Type: "BaseImage",
            },
            Style: {}
        }
        const item = new SPersagyImageEdit(null, data);
        item.selectable = true;
        item.moveable = true;
        this.addItem(item);
        this.undoStack.push(new SGraphAddCommand(this, item));
        this.Markers.push(item);
        this.finishCreated(item);
    }

    /**
     * 添加基本矩形item
     */
    addRectItem(event: SMouseEvent): void {
        const data = {
            /** ID */
            ID: uuid(),
            /** 名称  */
            Name: '基础矩形',
            /** 图标（Image），线类型(Line) */
            Type: "Zone",
            /** 位置  */
            Pos: { X: 0, Y: 0 },
            /** 由应用自己定义  */
            Properties: {
                Type: "BaseRect",
            },
            Style: {
                Line: [{ X: event.x, Y: event.y }],
            }
        }
        const rectItem = new SBaseRectEdit(null, data);
        rectItem.status = SItemStatus.Create;
        this.addItem(rectItem);
        this.undoStack.push(new SGraphAddCommand(this, rectItem));
        rectItem.selectable = true;
        this.grabItem = rectItem;
        rectItem.connect("finishCreated", this, this.finishCreated);
        rectItem.connect("onContextMenu", this, this.getItem);
        if (this.view) {
            this.view.update();
        }
    }

    /**
     * 添加基本三角形item
     */
    addTriangleItem(event: SMouseEvent): void {

    }

    /**
     * 添加基本箭头item
     */
    addArrowsItem(event: SMouseEvent): void {

    }

    /**
     * 修改 item 样式，数据等方法
     * @param styleType string 修改样式类型
     * @param changeStyle 更改样式数据
     * @param itemList? SGraphEdit[] 如果不传入默认使用选择器中选中得item
     */
    updateStyle(styleType: string, changeStyle: any, itemList?: SGraphEdit[]): void {
        // 如果未传入需要修改样式的item,默认取选择器中的item
        let List = null;
        if (itemList && itemList.length) {
            List = itemList;
        } else {
            List = this.selectContainer.itemList;
        };
        let styleValue: any
        if (styleType == "strokeColor") {
            const colorlist = rgbaNum(changeStyle)
            styleValue = new SColor(Number(colorlist[0]), Number(colorlist[1]), Number(colorlist[2]), Number(colorlist[3]));
        }
        List.forEach((item: SGraphEdit, index: number) => {
            if (item instanceof SGraphSelectContainer) {
                return
            }
            item[styleType] = styleValue
        })

    }

    /**
     * item 创建完成后回调
     *
     * @param   event   鼠标事件参数
     */
    finishCreated(item: SGraphEdit): void {
        this.grabItem = null;
        item.status = SItemStatus.Normal;
    }
    /**
     * 修改 cmdstatus 函数；常在在业务中调用
     *
     */
    clearCmdStatus() {
    }
    /**
     * 获取item (常用与场景外的调用F)
     * @param  event SMouseEvent 鼠标事件
     * @param  item SGraphEdit|null 返回item
     *
     */
    getItem(item: SGraphEdit | null, event: SMouseEvent[]) {
    }
    /**
     * 获取item (常用与场景外的调用F)
     * @param  event SMouseEvent 鼠标事件
     * @param  item SGraphEdit|null 返回item
     *
     */
    vueOnMouseDown(event: SMouseEvent) {
    }

    /**
     * 获取item (常用与场景外的调用F)
     * @return obj 返回保存的数据接口
     */
    save() {
        if (!this.view) return;
        const MarkType: string[] = ['BaseLine', 'BaseText', 'BaseExplain', 'BaseImage'];
        const Markers: any = [];   /**图例节点 */  // 与工程信息无关的标识对象（增加文本注释，图上的图片说明）
        const Nodes: any = [];   /**图例节点 */  // 与工程信息无关的标识对象（增加文本注释，图上的图片说明）
        const Relations: any = [];   /**图例节点 */  // 与工程信息无关的标识对象（增加文本注释，图上的图片说明）
        console.log('this.root.children', this.root.children)
        this.root.children.forEach(item => {
            if (!(item instanceof SGraphSelectContainer) && MarkType.includes(item.data.Properties.Type)) {
                Markers.push(item.toData())
            }
        });
        return {
            Markers,
            Nodes,
            Relations
        }
    }
}